import objectdraw.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.jfugue.*;

/*
 * A NoisyButton is a GUI component designed for the implementation of the
 * game Simon.  On the screen, a NoisyButton appears as a colored square.
 * When the user clicks on a NoisyButton, a "listener" is notifed by invoking
 * its "noisyButtonClicked" method.  A "flash" method is provided which
 * briefly lightens the color of the button and makes a noise.
 * 
 * Original by Williams College Computer Science faculty
 * Modified by Jim Teresco, CSC 252, The College of Saint Rose, Fall 2014
 * 
 * $Id: NoisyButton.java 2477 2014-11-10 02:54:02Z terescoj $
 *
 */

public class NoisyButton extends JPanel implements MouseListener {
    
    // whether to play any noise or not
    private static final boolean PLAY_NOISE = true;
    
    private static final int PAUSETIME = 30; // Duration of flash
    private static final int BUTTONSIZE = 300; // The size of the button square
    private static final int NOTETIME = 300;
    
    // The name of the instrument to use in the synthesizer
    //  see http://www.jfugue.com for more options
    private static final String INSTRUMENT = "I[CONTRABASS]";
    
    // The note to play for the nasty noise
    private static final String RAZZ_NOTE = " F3+E3+D3+C3+B3+A3";
    
    // The duration.  w=whole,h=half,i=eight,q=quarter
    private static final String DURATION = "i";
    
    // The JFugue synthesizer player.  One instance is shared
    //  among all the buttons (hence, the "static" qualifier).
    private static Player player = new Player();
    
    private JDrawingCanvas buttonArea; // Canvas on which the button is drawn
    private FilledRect coloring;       // Rectangle drawn to make the button
    private Color shade;               // color of button
    
    // Sound to play when the button flashes
    //  should be "A", "B", ...  
    // See http://www.jfugue.com for more options
    private String note; 
    
    // lock to ensure that only one button can beep at a time.
    private static Object buttonLock = new Object();
    
    // Object to notify when button is clicked
    private NoisyButtonListener listener;
    
    // construct a new NoisyButton
    //     noise determines the tone played when the button is clicked
    //     shade determines the unhighlighted color of the button
    public NoisyButton(String note, Color shade) {
        this.shade = shade;
        this.note = note;
        
        // create a canvas for the button, add it to the Panel and listen for clicks
        buttonArea = new JDrawingCanvas(BUTTONSIZE, BUTTONSIZE);
        add(buttonArea);
        buttonArea.addMouseListener(this);
        
        // draw a rectangle to make button appear
        coloring = new FilledRect(0, 0, BUTTONSIZE, BUTTONSIZE, buttonArea);
        coloring.setColor(shade);
        
    }
    
    // Assign an object to listen for when someone clicks on the button
    public void addListener(NoisyButtonListener listener) {
        
        this.listener = listener;
    }
    
    // If someone presses on the canvas, inform listener that they clicked on the button
    public void mousePressed(MouseEvent e) {
        
        if (listener != null)
            listener.noisyButtonClicked(this);
    }
    
    //
    // Other unneeded methods required to satisfy "MouseListener" interface
    //
    public void mouseEntered(MouseEvent e) { }
    
    public void mouseExited(MouseEvent e) { }
    
    public void mouseClicked(MouseEvent e) { }
    
    public void mouseReleased(MouseEvent e) { }
    
    // Make the button flash creating an ActiveObject to pause if necessary
    public void flash() {
        
        if ( (Thread.currentThread()) instanceof ActiveObject ) {
            realFlash();
        } else {
            new Beeper(this);
        }   
    }
    
    
    // make the button flash
    private void realFlash(){
        
        // make sure only one button can be flashed at a time
        synchronized (buttonLock) {
            coloring.setColor(shade.brighter().brighter());
            if (PLAY_NOISE) {               
                player.play(new Pattern(INSTRUMENT + " " +note + DURATION));
                // Wait for the player to be done playing before continuing
                // This busy waits, but that should be ok since it will only
                // wait for a few millis most of the time.
                while (player.getSequencer().isRunning()) {}
            }
            else {
                pause(NOTETIME);
            }
            coloring.setColor(shade);
            // The player is synchronous, but we need the
            // pause here to avoid a timing problem where
            // the player stops playing notes for a few seconds
            pause(PAUSETIME);
        }
    }
    
    private static void pause(long delay) {
        
        try{ 
            Thread.sleep(delay);
        } 
        catch (Exception e) {}
        
    }
    
    
    // Play a "nasty" noise.  This method is static,
    // so to call it, use the following command:
    //
    //    NoisyButton.playNastyNoise();
    //
    public static void playNastyNoise() {

        realPlayNastyNoise();
    }
    
    public static void realPlayNastyNoise() {
        
        synchronized (buttonLock) {
            if (PLAY_NOISE) {               
                player.play(new Pattern(INSTRUMENT + RAZZ_NOTE));
                // Wait for the player to be done playing before continuing
                // This busy waits, but that should be ok since it will only
                // wait for a few millis most of the time.
                while (player.getSequencer().isRunning());
                // The player is synchronous, but we need the
                // pause here to avoid a timing problem where
                // the player stops playing notes for a few seconds
                pause(10 * PAUSETIME);
            }
            else {
                JOptionPane.showMessageDialog(null, "WRONG!!!");
                pause(10 * PAUSETIME);
            }
        }
    }
}

// An object of the class Beeper is created each time the button must be flashed.
// The execute method of the object simply changes the button's color and plays its
// sound, then pauses and changes its color back.
class Beeper extends ActiveObject {
    private NoisyButton toBeep;
    
    // create a beeper
    public Beeper( NoisyButton beepMe){
        toBeep = beepMe;
        start();
    }
    
    public void run() {     
        toBeep.flash();
    }
}
